package org.apollo;

import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.util.List;

import org.apollo.widgets.LinkedTrack;
import org.apollo.widgets.SampledTrack;
import org.expeditee.gui.DisplayIO;
import org.expeditee.gui.Frame;
import org.expeditee.gui.FrameGraphics;
import org.expeditee.gui.FrameMouseActions;
import org.expeditee.gui.FreeItems;
import org.expeditee.items.Item;
import org.expeditee.items.widgets.InteractiveWidget;

/**
 * Apollos mouse actions extensions. Overrides expeditees frame mouse actions and
 * forwards them onto expeditee (filtering out certain non-conflicting key-sequences).
 * 
 * @author Brook Novak
 *
 */
public class AudioFrameMouseActions implements MouseMotionListener {
	
	private static final int COARSE_PLACEMENT_TOLERANCE = 100; // The minium distance from a track widet to auto-align free space items to

	public void mouseDragged(MouseEvent e) {
		FrameMouseActions.getInstance().mouseDragged(e);
	}

	public void mouseMoved(MouseEvent e) {

		boolean forwareToExpeditee = true;
		
		if (AudioFrameKeyboardActions.isControlDown != e.isControlDown()) {
			AudioFrameKeyboardActions.isControlDown = e.isControlDown();
			FrameGraphics.refresh(false); // For special post effects
		}
		
		AudioFrameKeyboardActions.isShiftDown = e.isShiftDown();
		
		if (isYAxisRestictionOn() && !FreeItems.getInstance().isEmpty()) { 

			// Restrict movement of free items to Y-Axis only
			forwareToExpeditee = false;

			int smallestY = FreeItems.getInstance().get(0).getY();

			for (Item i : FreeItems.getInstance()) {
				if (i.getY() < smallestY) smallestY = i.getY();
			}

			for (Item i : FreeItems.getInstance()) {
				i.setY(e.getY() + (i.getY() - smallestY));
			}
			
			// Avoids no-op suppression in expeditee once anchored free items
			FrameMouseActions.MouseX = e.getX();
			FrameMouseActions.MouseY = e.getY();
			
		} else if (e.isShiftDown() && !AudioFrameKeyboardActions.isControlDown &&
				!FreeItems.getInstance().isEmpty()) { 
			
			// Couse movement of free items: Restraining left-most pixel in items
			// to the closest anchored track widgets x position.
			
			Frame currentFrame = DisplayIO.getCurrentFrame();
			if (currentFrame != null) {
				
				// Search all anchored track x positions for the current frame
				List<InteractiveWidget> widgets = currentFrame.getInteractiveWidgets();
				
				int closestXPosition = -1;
				int closestDistance = -1;
				
				for (InteractiveWidget iw : widgets) {
					if (iw instanceof SampledTrack || iw instanceof LinkedTrack) {
						int distance = Math.abs(e.getX() - iw.getX());
						if (closestDistance < 0 || distance < closestDistance) {
							closestDistance = distance;
							closestXPosition = iw.getX();
						}

						distance = Math.abs(e.getX() - (iw.getX() + iw.getWidth()));
						if (closestDistance < 0 || distance < closestDistance) {
							closestDistance = distance;
							closestXPosition = iw.getX() + iw.getWidth();
						}
						
					}
				}
				
				int smallestX = FreeItems.getInstance().get(0).getX();
				int smallestY = FreeItems.getInstance().get(0).getY();

				for (Item i : FreeItems.getInstance()) {
					if (i.getX() < smallestX) smallestX = i.getX();
					if (i.getY() < smallestY) smallestY = i.getY();
				}
				
				// Should auto-adjust free items?
				if (closestDistance > 0 &&
						closestDistance < COARSE_PLACEMENT_TOLERANCE) { // NB: 0 = do nothing since already there

					for (Item i : FreeItems.getInstance()) {
						i.setPosition(
								closestXPosition + (i.getX() - smallestX), 
								e.getY() + (i.getY() - smallestY));
						
					}
					
				} else if (closestDistance > 0) { // keep things set to cursor

					for (Item i : FreeItems.getInstance()) {
						i.setPosition(
								e.getX() + (i.getX() - smallestX), 
								e.getY() + (i.getY() - smallestY));
					}

				}
				
				forwareToExpeditee = false;

			}
			
		}
		
		// Expeditees frame mouse actions uses an offset and fights over free-item
		// movement if it listens to the mouse event router... therefore add an extra
		// layer to avoid this... otherwise auto-aligned items jitter like crazy while
		// moving the cursus
		if (forwareToExpeditee) {
			FrameMouseActions.getInstance().mouseMoved(e);
		} else {
			FrameGraphics.refresh(true);
		}
	}
	
	/**
	 * @return
	 * 		True if the user is restricting movement on the y-axis only
	 */
	public static boolean isYAxisRestictionOn() {
		return AudioFrameKeyboardActions.isControlDown && !AudioFrameKeyboardActions.isShiftDown;
	}
	
	
}
